<?php
/**
 * WIKINDX : Bibliographic Management system.
 * @link http://wikindx.sourceforge.net/ The WIKINDX SourceForge project
 * @author The WIKINDX Team
 * @copyright 2017-2019 Mark Grimshaw <sirfragalot@users.sourceforge.net>
 * @license https://creativecommons.org/licenses/by-nc-sa/2.0/legalcode CC-BY-NC-SA 2.0
 */

/**
* WEBSERVERCONFIG
*
* CONFIGURE the webserver with default parameters
* compliant with Wikindx before applying
* its adminuser custom configuration.
*
* This have to be included at the top of each independant script
* (index.php, rss.php, plugins, dialogs, etc).
*
* We intend to provide correct default values for anything
* that is not very interesting for the end user.
*
* @version 1
*
* @package wikindx5\core\startup
* @author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/


/**
* Import CONSTANTS
*/
include_once("core/startup/CONSTANTS.php");


/**
* On startup, reports all errors.
* After that, error reporting is reconfigured
* by core/startup/LOADCONFIG.php (first called in AUTHORIZE) with user config.
* This prevent to hide tricky errors.
*/
error_reporting(WIKINDX_PHP_ERROR_REPORTING_DEFAULT);
ini_set('display_startup_errors', 'On');
ini_set('display_errors', WIKINDX_PHP_DISPLAY_ERRORS_DEFAULT);
ini_set('html_errors', 'On');

/**
* Fix default charset of PHP interpret, PHP libs and protocols
*/

// GLOBAL config for PHP, cascaded for some libs
ini_set('default_charset', WIKINDX_CHARSET);

// HTTP charset (HTTP specification doesn't permit to declare Content-type separetly)
header('Content-type: ' . WIKINDX_HTTP_CONTENT_TYPE_DEFAULT . '; charset=' . WIKINDX_CHARSET);

// mbstring lib
ini_set('mbstring.encoding_translation', WIKINDX_CHARSET);
ini_set('mbstring.detect_order', WIKINDX_MBSTRING_DETECT_ORDER);
mb_regex_encoding(WIKINDX_CHARSET);

// For PHP 5.6.0 and later, mbstring take the values below from default_charset
if (version_compare(phpversion(), '5.6.0', '<'))
{
    ini_set('mbstring.internal_encoding', WIKINDX_CHARSET); // Remove this line when Wikindx will need PHP 5.6.0 or later
    ini_set('mbstring.http_input', WIKINDX_CHARSET);
    ini_set('mbstring.http_output', WIKINDX_CHARSET);
}


// make sure that Session output is XHTML conform ('&amp;' instead of '&')
ini_set('arg_separator.output', '&amp;');

// Disable a syntax uncompatible with XML
ini_set('short_open_tag', 'Off');

// Disable PHP arg_v, argc, shebang
ini_set('register_argc_argv', 'Off');
ini_set('cgi.check_shebang_line', 'Off');

//TODO : this option can only be activated when the plugins
//       start the session after the inclusion of this code.
// Disallow access to session cookie by JavaScript
// This setting prevents cookies stolen by JavaScript injection
//ini_set('session.cookie_httponly', 'On');

//TODO : this option can only be activated when the plugins
//       start the session after the inclusion of this code.
// Session module only accepts valid session ID generated by session module
// It rejects session ID supplied by user forgery
//ini_set('session.use_strict_mode', 'On');


// Configure a default timezone if empty
if (date_default_timezone_get() == '') date_default_timezone_set(WIKINDX_TIMEZONE_DEFAULT);


// Check PHP minimum version and above.
if(version_compare(PHP_VERSION, WIKINDX_PHP_VERSION_MIN, '<'))
{
    $AppName = WIKINDX_NAME;
    $PHPVersion = PHP_VERSION;
    $PHPVersionMin = WIKINDX_PHP_VERSION_MIN;
    $msg =<<<EOM
<!DOCTYPE html>
<html lang="en">
<head>
	<title>$AppName</title>
	<meta charset="UTF-8">
	<link rel="stylesheet" href="templates/default/template.css" type="text/css">
	<link rel="shortcut icon" type="image/x-icon" href="templates/default/images/favicon.ico">
</head>
<body>

<h1>Configuration error: PHP outdated</h1>

<p>$AppName requires PHP <strong>$PHPVersionMin</strong> or greater.  Your PHP version is <em>$PHPVersion</em>. Please upgrade PHP.</p>

<p>After that, refresh this page (with Ctrl+F5) or <a href="index.php">follow this link</a>.</p>

</body>
EOM;
    die($msg);
}


// Check if mandatories extensions are enabled.
$MandatoryExtensions = array('Core', 'date', 'fileinfo', 'gd', 'json', 'mbstring', 'mysqli', 'session', 'SimpleXML');
$InstalledExtensions = get_loaded_extensions();
$MissingExtensions = array_diff($MandatoryExtensions, $InstalledExtensions);

if (count($MissingExtensions) > 0)
{
    $AppName = WIKINDX_NAME;
    $EnabledExtensions = array_intersect($MandatoryExtensions, $InstalledExtensions);
    $ListExtensions = '<tr><td>' . implode('</td><td style="color:red">DISABLED</td></tr><tr><td>', $MissingExtensions) . '<td style="color:red">DISABLED</td></tr>';
    $ListExtensions .= '<tr><td>' . implode('</td><td style="color:green">ENABLED</td></tr><tr><td>', $EnabledExtensions) . '<td style="color:green">ENABLED</td></tr>';
    $msg =<<<EOM
<!DOCTYPE html>
<html lang="en">
<head>
	<title>$AppName</title>
	<meta charset="UTF-8">
	<link rel="stylesheet" href="templates/default/template.css" type="text/css">
	<link rel="shortcut icon" type="image/x-icon" href="templates/default/images/favicon.ico">
</head>
<body>

<h1>Configuration error: missing PHP extensions</h1>

<p>$AppName requires the following PHP extensions to work properly :</p>

<table style="border: 1px solid black; width:33%">
<tr style="border: 1px solid black;">
    <th>Extension</th>
    <th>Status</th>
</tr>
    $ListExtensions
</table>

<p>Please install and enable them.</p>

<p>After that, refresh this page (with Ctrl+F5) or <a href="index.php">follow this link</a>.</p>

</body>
EOM;
    die($msg);
}

// Check PHP execution environnement (CLI isn't supported)
if (PHP_SAPI === 'cli') die(WIKINDX_NAME . " doesn't support CLI execution.");

// Check for presence of config.php
if(!is_file('config.php')) die(file_get_contents('page_config_file_missing.html'));

// Bufferize output
ob_start();

// Begin page execution timer and define globals for rendering by template
include_once("core/startup/GLOBALS.php");

// Set up the FACTORY objects of commonly used classes and start the timer.
include_once("core/startup/FACTORY.php");
include_once("core/startup/ENVIRONMENT.php");

// Init user config object
//$config = FACTORY_CONFIG::getInstance(); // not needed here (and interferes with upgrade of v3.8 database because v3 config.php is not a class

/**
*	Initialize the system
*	As ENVIRONMENT starts the session, we must check for 'remember me' cookie requests from MYWIKINDX.php prior to
*	setting the session.
*/
$env = new ENVIRONMENT();

$vars = GLOBALS::getVars();
if(!empty($vars) && array_key_exists('cookie', $vars) && $vars['cookie'] &&
    array_key_exists('uname', $vars) && trim($vars['uname']))
{
// set the cookie if requested
    $cookie = FACTORY_COOKIE::getInstance();
    $cookie->storeCookie($vars['uname']);
    unset($cookie);
}
// start the session
$env->startSession();

// Set the current working directory -- useful for ensuring TinyMCE plug-ins can find the wikindx5 base path for include() commands.
// Not all OSs allow getcwd() or sometimes the wikindx installation is in a directory that is not searchable.
if(!$wikindxBasePath = dirname(__FILE__))
{
//			$session->setVar('wikindxBasePath', $session->getVar('wikindxBasePath', dirname(__FILE__)));
	if(!isset($env->config->WIKINDX_WIKINDX_PATH) || !$env->config->WIKINDX_WIKINDX_PATH)
		die("WIKINDX is unable to set the installation path automatically.  You should set \$WIKINDX_WIKINDX_PATH in config.php");
	$path = $env->config->WIKINDX_WIKINDX_PATH;
// test path is correct
	if(!is_file($path . '/core/startup/LOADCONFIG.php'))
		die("\$WIKINDX_WIKINDX_PATH in config.php is set incorrectly");
	else
		$wikindxBasePath = $path;
}
$remove = preg_quote(DIRECTORY_SEPARATOR . 'core' . DIRECTORY_SEPARATOR . 'startup', '/');
$_SESSION['wikindxBasePath'] = preg_replace('/' . $remove . '$/', '', $wikindxBasePath);

// Load auth object but diff. login after upgrade stage
// Upgrade will request login to superadmin if needed
$authorize = FACTORY_AUTHORIZE::getInstance();
// Attempt an upgrade only if we are on the main script
if (mb_strripos('plugins/', $_SERVER['SCRIPT_NAME']) === FALSE)
{
    // Do database upgrade check
	include_once("core/startup/CHECKUPDATEDATABASE.php");
	$checkUpdate = new CHECKUPDATEDATABASE();
	if ($checkUpdate->needUpdate())
	{
	    // Upgrade database
	    include_once("core/startup/UPDATEDATABASE.php");
	    $update = new UPDATEDATABASE(); // __construct() runs on autopilot
	    $upgradeCompleted = $update->upgradeCompleted;
	    unset($update);
	}
	unset($checkUpdate);
	unset($env);
}

if(array_key_exists('action', $vars) && ($vars['action'] == 'continueExecution'))
    unset($vars['action']);
?>